==> 0.0.2.tar.gz: lib/controls/combination_picker.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:select_form_field/select_form_field.dart';
//import 'package:flutter_gen/gen_l10n/app_localizations.dart';

import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/services/datasource.dart';
import 'package:bizdoc/services/cube.dart';
import 'package:bizdoc/models/profile.dart';

typedef AxesArgs = void Function(
    Map<String, dynamic> axes, String? description, String? combination);

class CombinationPicker extends StatefulWidget {
  final String? labelText;
  final String? initialValue;
  final AxesArgs onChanged;
  final String? cube;
  const CombinationPicker(
      {Key? key,
      this.cube,
      required this.onChanged,
      this.labelText,
      this.initialValue})
      : super(key: key);
  @override
  _CombinationPickerState createState() => _CombinationPickerState();
}

class _CombinationPickerState extends State<CombinationPicker> {
  final Map<String, dynamic> _values = {};
  final Map<String, Map<String, dynamic>> _datasources = {};

  late CubeInfo cubeInfo;
  late Iterable<CubeAxis> axes;

  @override
  void initState() {
    cubeInfo = session.profile.cubes.firstWhere(
        (element) => widget.cube == null || element.name == widget.cube);
    axes = this.cubeInfo.axes.where((element) => element.combination);
    super.initState();
  }

  _initialize() async {
    final values =
        widget.initialValue != null ? widget.initialValue!.split('.') : null;
    for (var i = 0; i < axes.length; i++) {
      var axis = axes.elementAt(i);
      var data =
          _datasources[axis.name] = await dataSource.getMap(axis.dataType);
      var firstValue = data.length == 1 ? data.entries.first.key : null;
      var initialValue = values?.elementAt(i) ?? firstValue;
      if (initialValue != null) _values[axis.name] = initialValue;
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: _initialize(),
      builder: (context, snapshot) => Column(
        children: [
          for (var axis in axes)
            _datasources[axis.name]!.length < 10
                ? SelectFormField(
                    decoration: InputDecoration(labelText: axis.title),
                    type: SelectFormFieldType.dropdown,
                    initialValue: _values.containsKey(axis.name)
                        ? _values[axis.name]
                        : null,
                    enableSearch: true,
                    items: _items(axis.name),
                    labelText: axis.title,
                    onChanged: (value) => _change(axis.name, value),
                  )
                : TypeAheadField<MapEntry>(
                    textFieldConfiguration: TextFieldConfiguration(
                      controller: TextEditingController(
                          text: _values.containsKey(axis.name)
                              ? _datasources[axis.name]![_values[axis.name]]
                              : null),
                      // decoration: InputDecoration(
                      //   labelText: AppLocalizations.of(context)!.project,
                      // )
                    ),
                    suggestionsCallback: (value) => _datasources[axis.name]!
                        .entries
                        .where((e) =>
                            e.key.startsWith(value) ||
                            e.value.toString().contains(value)),
                    itemBuilder: (context, item) => ListTile(
                      title: Text(item.value),
                      subtitle: Text(item.key),
                    ),
                    onSuggestionSelected: (item) {
                      _change(axis.name, item.key);
                    },
                  ),
        ],
      ),
    );
  }

  List<Map<String, dynamic>> _items(String name) {
    var items = _datasources[name]!
        .entries
        .map((e) => {'value': e.key, 'label': e.value})
        .toList();
    // items.add({'value': null, 'label': AppLocalizations.of(context)!.none});
    return items;
  }

  _change(String axis, String? value) async {
    if (value == null) {
      _values.remove(axis);
    } else {
      _values[axis] = value;
    }
    try {
      final constraints = await cube.constraints(cubeInfo.name, axes: _values);
      setState(() {
        for (var element in constraints.keys) {
          _datasources[element] = constraints[element]!;
        }
      });
    } catch (e) {
      print(e);
    }
    if (_values.length == axes.length) {
      widget.onChanged(
          _values,
          axes.map((e) => _datasources[e.name]![_values[e.name]]).join('.'),
          axes.map((e) => _values[e.name]).join('.'));
    }
  }
}

==> 0.0.4.tar.gz: lib/controls/combination_picker.dart <==
library bizdoc;

import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:select_form_field/select_form_field.dart';
//import 'package:flutter_gen/gen_l10n/app_localizations.dart';

import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/services/datasource.dart';
import 'package:bizdoc/services/cube.dart';
import 'package:bizdoc/models/profile.dart';

typedef AxesArgs = void Function(
    Map<String, dynamic> axes, String? description, String? combination);

class CombinationPicker extends StatefulWidget {
  final String? labelText;
  final String? initialValue;
  final AxesArgs onChanged;
  final String? cube;
  const CombinationPicker(
      {Key? key,
      this.cube,
      required this.onChanged,
      this.labelText,
      this.initialValue})
      : super(key: key);
  @override
  _CombinationPickerState createState() => _CombinationPickerState();
}

class _CombinationPickerState extends State<CombinationPicker> {
  final Map<String, dynamic> _values = {};
  final Map<String, Map<String, dynamic>> _datasources = {};

  late CubeInfo cubeInfo;
  late Iterable<CubeAxis> axes;

  @override
  void initState() {
    cubeInfo = session.profile.cubes.firstWhere(
        (element) => widget.cube == null || element.name == widget.cube);
    axes = this.cubeInfo.axes.where((element) => element.combination);
    super.initState();
  }

  _initialize() async {
    final values =
        widget.initialValue != null ? widget.initialValue!.split('.') : null;
    for (var i = 0; i < axes.length; i++) {
      var axis = axes.elementAt(i);
      var data = _datasources[axis.name] = await dataSource.map(axis.dataType);
      var firstValue = data.length == 1 ? data.entries.first.key : null;
      var initialValue = values?.elementAt(i) ?? firstValue;
      if (initialValue != null) _values[axis.name] = initialValue;
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: _initialize(),
      builder: (context, snapshot) => Column(
        children: [
          for (var axis in axes)
            _datasources[axis.name]!.length < 10
                ? SelectFormField(
                    decoration: InputDecoration(labelText: axis.title),
                    type: SelectFormFieldType.dropdown,
                    initialValue: _values.containsKey(axis.name)
                        ? _values[axis.name]
                        : null,
                    enableSearch: true,
                    items: _items(axis.name),
                    labelText: axis.title,
                    onChanged: (value) => _change(axis.name, value),
                  )
                : TypeAheadField<MapEntry>(
                    textFieldConfiguration: TextFieldConfiguration(
                      controller: TextEditingController(
                          text: _values.containsKey(axis.name)
                              ? _datasources[axis.name]![_values[axis.name]]
                              : null),
                      // decoration: InputDecoration(
                      //   labelText: AppLocalizations.of(context)!.project,
                      // )
                    ),
                    suggestionsCallback: (value) => _datasources[axis.name]!
                        .entries
                        .where((e) =>
                            e.key.startsWith(value) ||
                            e.value.toString().contains(value)),
                    itemBuilder: (context, item) => ListTile(
                      title: Text(item.value),
                      subtitle: Text(item.key),
                    ),
                    onSuggestionSelected: (item) {
                      _change(axis.name, item.key);
                    },
                  ),
        ],
      ),
    );
  }

  List<Map<String, dynamic>> _items(String name) {
    var items = _datasources[name]!
        .entries
        .map((e) => {'value': e.key, 'label': e.value})
        .toList();
    // items.add({'value': null, 'label': AppLocalizations.of(context)!.none});
    return items;
  }

  _change(String axis, String? value) async {
    if (value == null) {
      _values.remove(axis);
    } else {
      _values[axis] = value;
    }
    try {
      final constraints = await cube.constraints(cubeInfo.name, axes: _values);
      setState(() {
        for (var element in constraints.keys) {
          _datasources[element] = constraints[element]!;
        }
      });
    } catch (e) {
      print(e);
    }
    if (_values.length == axes.length) {
      widget.onChanged(
          _values,
          axes.map((e) => _datasources[e.name]![_values[e.name]]).join('.'),
          axes.map((e) => _values[e.name]).join('.'));
    }
  }
}
